/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>


/* The default #of objects to put/get/delete into/from the hash tables */
#define DEFAULT_OBJS 9876


/* Limits for static buffers in utoa() */
#define ITEMS 10      /* rows */
#define SIZE  16      /* cols */


/* Item definition */
typedef struct ritem ritem_t;
struct ritem
{
  unsigned   u;
  char     * s;
  ritem_t  * p;

  ritem_t  * prev;    /* needed for a doubly-linked list only */
  ritem_t  * next;    /* needed for singly or doubly-linked lists */

};


/* Object definition */
typedef struct robj
{
  unsigned      ukey;
  char        * skey;
  struct robj * self;

  unsigned      uval;

} robj_t;


typedef struct
{
  unsigned int num;
  const char * str;
} num_str_t;
 

num_str_t my_map [] =
{
  { .str = "zero",  .num = 0 },
  { .str = "one",   .num = 1 },
  { .str = "two",   .num = 2 },
  { .str = "three", .num = 3 },
  { .str = "four",  .num = 4 },
  { .str = "five",  .num = 5 },
  { .str = "six",   .num = 6 },
  { .str = "seven", .num = 7 },
  { .str = "eight", .num = 8 },
  { .str = "nine",  .num = 9 }
};


/*
 * MurmurHash3
 * https://github.com/aappleby/smhasher
 */
uint32_t MurmurHash3 (uint32_t h)
{
  h ^= h >> 16;
  h *= 0x85ebca6b;
  h ^= h >> 13;
  h *= 0xc2b2ae35;
  h ^= h >> 16;
  return h;
}


int cmp_f (robj_t * a, robj_t * b)
{
  return a -> ukey - b -> ukey;
}


uint32_t hash_f (robj_t * obj)
{
  return MurmurHash3 (obj -> ukey);
}


/* Unsigned to Ascii */
char * utoa (unsigned n)
{
  static char text [ITEMS] [SIZE];
  static unsigned i = 0;

  char * s = text [i ++ % ITEMS];
  sprintf (s, "%u", n);
  return s;
}


ritem_t * mkitem (unsigned u)
{
  ritem_t * ptr = calloc (1, sizeof (* ptr));

  ptr -> u = u;
  ptr -> s = strdup (utoa (u));
  ptr -> p = ptr;
  return ptr;
}


void rmitem (ritem_t * item)
{
  if (! item)
    return;
  if (item -> s)
    free (item -> s);
  free (item);
}


robj_t * mkobj (unsigned key, unsigned val)
{
  robj_t * ptr = calloc (1, sizeof (* ptr));

  ptr -> ukey = key;
  ptr -> skey = strdup (utoa (key));
  ptr -> self = ptr;
  ptr -> uval = val * 2;
  return ptr;
}


robj_t ** mkobjargv (unsigned argc)
{
  robj_t ** argv = calloc (argc, sizeof (* argv));
  unsigned i;

  for (i = 0; i < argc; i ++)
    argv [i] = mkobj (i + 1, i + 1);

  return argv;
}


robj_t ** mkobjs (unsigned argc)
{
  return mkobjargv (argc);
}


void rmobj (robj_t * obj)
{
  if (! obj)
    return;
  if (obj -> skey)
    free (obj -> skey);
  free (obj);
}


void rmobjargv (robj_t * argv [], unsigned argc)
{
  unsigned i;

  for (i = 0; i < argc; i ++)
    rmobj (argv [i]);
  free (argv);
}


void rmobjs (robj_t * argv [], unsigned argc)
{
  rmobjargv (argv, argc);
}
